@@ -218,7 +218,7 @@ lmtp_rc_t lmtp_readmsg(lmtp_t *lmtp, char **cppBuf, size_t maxlen)
     size_t offset;     /* required when cpBuf changed through realloc */
     size_t offsetline; /* memorizing start of line when reallocing in the middle of a line */
 
-    nBuf = 4096;
+    for (nBuf = 4096; nBuf > maxlen; nBuf = nBuf >> 1);
     if ((cpBuf = (char *)malloc(nBuf)) == NULL)
         return LMTP_ERR_MEM;
     *cppBuf = cpBuf;                           /* tell caller about the buffer */
@@ -320,18 +320,18 @@ lmtp_rc_t lmtp_response(lmtp_t *lmtp, lmtp_res_t *res)
     char formatbuf[LMTP_LINE_MAXLEN];
 
     if (   strlen(res->statuscode) != 3
-        || !isdigit(res->statuscode[0])
-        || !isdigit(res->statuscode[1])
-        || !isdigit(res->statuscode[2]))
+        || !isdigit((int)res->statuscode[0])
+        || !isdigit((int)res->statuscode[1])
+        || !isdigit((int)res->statuscode[2]))
         return LMTP_ERR_ARG;
 
     if (res->dsncode != NULL) {
         if (   (strlen(res->dsncode) != 5)
-            || !isdigit(res->dsncode[0])
+            || !isdigit((int)res->dsncode[0])
             || (res->dsncode[1] != '.')
-            || !isdigit(res->dsncode[2])
+            || !isdigit((int)res->dsncode[2])
             || (res->dsncode[3] != '.')
-            || !isdigit(res->dsncode[4])
+            || !isdigit((int)res->dsncode[4])
             || (res->dsncode[0] != res->statuscode[0]))
             return LMTP_ERR_ARG;
     }
@@ -346,15 +346,23 @@ lmtp_rc_t lmtp_response(lmtp_t *lmtp, lmtp_res_t *res)
             len = sprintf(formatbuf, "%3.3s%c%5.5s ", res->statuscode, dash ? '-' : ' ', res->dsncode);
         else
             len = sprintf(formatbuf, "%3.3s%c", res->statuscode, dash ? '-' : ' ');
+        if ((len + cpE - cpS + 2) > sizeof(formatbuf)) { /* status + line + '\r\n' does not fit into formatbuf */
+            dash = 1;
+            if ((cpE = cpS + sizeof(formatbuf) - 2 - len) <= cpS) /* no space for line at all */
+                return LMTP_ERR_ARG;
+        }
         strncpy(formatbuf+len, cpS, cpE-cpS);
         len += (cpE-cpS);
+        formatbuf[len++] = '\r';
         formatbuf[len++] = '\n';
         do {
             rv = lmtp->io.write(lmtp->wfd, formatbuf, len);
         } while (rv == -1 && errno == EINTR);
         if (rv == -1)
             return LMTP_ERR_SYSTEM;
-        cpS = cpE+1;
+        cpS = cpE;
+        if (*cpS == '\n')
+            cpS++;
     }
     return rc;
 }
