@@ -16,6 +16,7 @@
 #include <string.h>
 #include <fcntl.h>
 #include <sys/utsname.h>
+#include <sys/time.h>
 
 /* third party */
 #include "str.h"
@@ -52,9 +53,8 @@
 #define ERR_EXECUTION -1
 #define ERR_DELIVERY -2
 
-#define MESSAGE_MAXLEN 8*1024*1024
 #define STDSTRLEN 128
-#define MAXNEWSSERVICES 3
+#define MAXNEWSSERVICES 16
 
 extern void lmtp_debug_dumplmtp(lmtp_t *lmtp);
 
@@ -74,6 +74,7 @@ struct session {
     char   *lhlo_domain;
 };
 
+static int connect_nonb(int, const struct sockaddr *, socklen_t, int);
 static void initsession(struct session *session);
 static void resetsession(struct session *session);
 int groupmatch(char *, size_t, char *);
@@ -94,6 +95,8 @@ typedef struct {
     int             option_deliverymode;
     char           *option_deliverymodefakestatus;
     char           *option_deliverymodefakedsn;
+    int             option_maxmessagesize;
+    int             option_waittime;
     int             nsc;
     struct ns       ns[MAXNEWSSERVICES];
     char           *azGroupargs;
@@ -125,13 +128,21 @@ enum {
 static void usage(char *command)
 {
     /*  use
-     *  perl <lmtp2nntp.c -e 'while (<>) { if(m/\/\*POD (.*) .*\*\//) { $_=$1; s/.<(.*?)>/$1/g ; print "$_ " };}'
+     *  perl <lmtp2nntp.c -e 'while (<>) { if(m/\/\*POD (.*) .*\*\//) { $_=$1; s/.<(.*?)>/$1/g ; print "\"$_\"\n" };}'
      *  to pull the USAGE string out of this source
      */
     fprintf(stderr, 
             "USAGE: %s "
-            "lmtp2nntp [-d deliverymode] [-g groupmode] [-h host[:port]] "
-            "[-t tracefile] [-v] [-V] newsgroup [newsgroup ...]"
+            "[-V]"
+            "[-d deliverymode]"
+            "[-g groupmode]"
+            "[-h host[:port][,host[:port], ...]]"
+            "[-m maxmessagesize]"
+            "[-n hostname]"
+            "[-t tracefile]"
+            "[-v]"
+            "[-w waittime]"
+            "newsgroup [newsgroup ...]"
             "\n",
             command);
     return;
@@ -163,6 +174,8 @@ int main(int argc, char **argv)
     ctx->option_deliverymode = DELIVERYMODE_FAKE;
     ctx->option_deliverymodefakestatus = "553";   /* Requested action not taken: mailbox name not allowed */
     ctx->option_deliverymodefakedsn    = "5.7.1"; /* Delivery not authorized, message refused */
+    ctx->option_maxmessagesize = 8 * 1024 * 1024;
+    ctx->option_waittime = -1;
     ctx->nsc = 0;
     for (i=0; i < MAXNEWSSERVICES; i++) {
         ctx->ns[i].h = NULL;
@@ -199,13 +212,17 @@ int main(int argc, char **argv)
     /*POD B<lmtp2nntp> */
 
     /*  use
-     *  perl <lmtp2nntp.c -e 'while (<>) { if(m/\/\*POD (.*) .*\*\//) { $_=$1; s/.<(.*?)>/$1/g ; print "$_\n" };}'
+     *  perl <lmtp2nntp.c -e 'while (<>) { if(m/\/\*POD (.*) .*\*\//) { $_=$1; print "$_\n" };}'
      *  to pull the POD SYNOPSIS header directly out of this source
      */
 
     /* read in the arguments */
-    while ((i = getopt(argc, argv, "d:g:h:t:vV")) != -1) {
+    while ((i = getopt(argc, argv, "Vd:g:h:m:n:t:vw:")) != -1) {
         switch (i) {
+            case 'V': /*POD [B<-V>] (version)*/
+                fprintf(stdout, "%s\n", lmtp2nntp_version.v_gnu);
+                exit(0);
+                break;
             case 'd': /*POD [B<-d> I<deliverymode>] */
                 if      (strcasecmp(optarg, "post") == 0)
                     ctx->option_deliverymode = DELIVERYMODE_POST;
@@ -258,7 +275,7 @@ int main(int argc, char **argv)
                     exit(ERR_EXECUTION);
                 }
                 break;
-            case 'h': /*POD [B<-h> I<host>[I<:port>]] */
+            case 'h': /*POD [B<-h> I<host>[I<:port>][,I<host>[I<:port>], ...]] */
                 if (argz_create_sep(optarg, ',', &azHosts, &asHosts) != 0)
                     exit(ERR_EXECUTION);
                 cp = NULL;
@@ -304,6 +321,20 @@ int main(int argc, char **argv)
                 }
                 free(azHosts);
                 break;
+            case 'm': /*POD [B<-m> I<maxmessagesize>] */
+                    ctx->option_maxmessagesize = atoi(optarg);
+                    if(ctx->option_maxmessagesize < 64) {
+                        fprintf(stderr, "%s:Error: maximum message size is unacceptable small.\n", progname);
+                        exit(ERR_EXECUTION);
+                    }
+                break;
+            case 'n': /*POD [B<-n> I<hostname>] */
+                if (strlen(optarg) > sizeof(ctx->uname.nodename)-1) {
+                        fprintf(stderr, "%s:Error: nodename \"%s\" to long to option -n.\n", progname, optarg);
+                        exit(ERR_EXECUTION);
+                }
+                strcpy(ctx->uname.nodename, optarg);
+                break;
             case 't': /*POD [B<-t> I<tracefile>] */
                 ctx->option_tracing = TRUE;
                 trace_read (-1, optarg, 0);
@@ -312,9 +343,13 @@ int main(int argc, char **argv)
             case 'v': /*POD [B<-v>] (verbose)*/
                 ctx->option_verbose = TRUE;
                 break;
-            case 'V': /*POD [B<-V>] (version)*/
-                fprintf(stdout, "%s\n", lmtp2nntp_version.v_gnu);
-                exit(0);
+            case 'w': /*POD [B<-w> I<waittime>] */
+                    ctx->option_waittime = atoi(optarg);
+                    if(ctx->option_waittime < 1) {
+                        fprintf(stderr, "%s:Error: waittime %d to option -w must be greater 1 second.\n", 
+                                progname, ctx->option_waittime);
+                        exit(ERR_EXECUTION);
+                    }
                 break;
             case '?':
             default:
@@ -374,6 +409,61 @@ int main(int argc, char **argv)
     return rc;
 }
 
+/* taken from "UNIX Network Programming", Volume 1, second edition W. Richard
+ * Stevens, connect_nonb.c from section 15.4 "Nonblocking connect", page 411,
+ * http://www.kohala.com/start/
+ */
+int connect_nonb(int sockfd, const struct sockaddr *saptr, socklen_t salen, int nsec)
+{
+	int				flags, n, error;
+	socklen_t		len;
+	fd_set			rset, wset;
+	struct timeval	tval;
+
+	flags = fcntl(sockfd, F_GETFL, 0);
+	fcntl(sockfd, F_SETFL, flags | O_NONBLOCK);
+
+	error = 0;
+	if ( (n = connect(sockfd, (struct sockaddr *) saptr, salen)) < 0)
+		if (errno != EINPROGRESS)
+			return(-1);
+
+	/* Do whatever we want while the connect is taking place. */
+
+	if (n == 0)
+		goto done;	/* connect completed immediately */
+
+	FD_ZERO(&rset);
+	FD_SET(sockfd, &rset);
+	wset = rset;
+	tval.tv_sec = nsec;
+	tval.tv_usec = 0;
+
+	if ( (n = select(sockfd+1, &rset, &wset, NULL,
+					 nsec ? &tval : NULL)) == 0) {
+		close(sockfd);		/* timeout */
+		errno = ETIMEDOUT;
+		return(-1);
+	}
+
+	if (FD_ISSET(sockfd, &rset) || FD_ISSET(sockfd, &wset)) {
+		len = sizeof(error);
+		if (getsockopt(sockfd, SOL_SOCKET, SO_ERROR, &error, &len) < 0)
+			return(-1);			/* Solaris pending error */
+	} else
+		return(-1); /* err_quit("select error: sockfd not set"); */
+
+done:
+	fcntl(sockfd, F_SETFL, flags);	/* restore file status flags */
+
+	if (error) {
+		close(sockfd);		/* just in case */
+		errno = error;
+		return(-1);
+	}
+	return(0);
+}
+
 static void resetsession(struct session *session)
 {
     if (session->lhlo_domain != NULL)
@@ -463,13 +553,24 @@ static lmtp_rc_t lmtp_cb_lhlo(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
     i = 0;
     do {
         bOk = TRUE;
-        if (connect(ctx->ns[i].s, ctx->ns[i].sa->sa_buf, ctx->ns[i].sa->sa_len) < 0) {
-            bOk = FALSE;
+        if (ctx->option_waittime > 0) {
+            if (connect_nonb(ctx->ns[i].s, ctx->ns[i].sa->sa_buf, ctx->ns[i].sa->sa_len, 
+                         ctx->option_waittime) < 0) {
+                bOk = FALSE;
+            }
+        }
+        else {
+            if (connect(ctx->ns[i].s, ctx->ns[i].sa->sa_buf, ctx->ns[i].sa->sa_len) < 0) {
+                bOk = FALSE;
+            }
         }
         if (bOk && ((ctx->ns[i].nntp = nntp_create(ctx->ns[i].s, ctx->ns[i].s, 
                                                    ctx->option_tracing ? &nntp_io : NULL)) == NULL)) {
             bOk = FALSE;
         }
+        if (bOk && ctx->option_waittime >= 0) {
+            nntp_timeout(ctx->ns[i].nntp, ctx->option_waittime);
+        }
         if (bOk && ((rc = nntp_init(ctx->ns[i].nntp)) != NNTP_OK)) {
             bOk = FALSE;
         }
@@ -873,16 +974,17 @@ static lmtp_rc_t lmtp_cb_data(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
     res.statusmsg  = "Enter mail, end with \".\" on a line by itself";
     lmtp_response(lmtp, &res);
 
-    rc = lmtp_readmsg(lmtp, &ctx->msg->cpMsg, MESSAGE_MAXLEN);
+    rc = lmtp_readmsg(lmtp, &ctx->msg->cpMsg, ctx->option_maxmessagesize);
 
     /*  RFC0821 4.2.1. REPLY CODES BY FUNCTION GROUPS   552 Requested mail action aborted: exceeded storage allocation
      *  RFC1893 2. Status Codes                         5.X.X   Permanent Failure
-     *  RFC1893 3.5 Network and Routing Status          X.3.4   Message too big for system
+     *  RFC1893 3.5 Network and Routing Status          X.2.3   Message length exceeds administrative limit.
+
      */
     if (rc == LMTP_ERR_OVERFLOW) {
-        str_format(errorstring, sizeof(errorstring), "Overflow reading message: %s", lmtp_error(rc));
+        str_format(errorstring, sizeof(errorstring), "Message length exceeds administrative limit. %s", lmtp_error(rc));
         res.statuscode = "552";
-        res.dsncode    = "5.3.4";
+        res.dsncode    = "5.2.3";
         res.statusmsg  = errorstring;
         rcpt = NULL;
         while ((rcpt = argz_next(ctx->msg->azRcpt, ctx->msg->asRcpt, rcpt)) != NULL) {
