@@ -25,7 +25,7 @@ typedef struct {
 typedef enum {
     MSG_OK,
     MSG_ERR_MEM,
-    MSG_ERR_SPLITSPLITBODY,
+    MSG_ERR_SPLITHEADBODY,
     MSG_ERR_SPLITLEN,
     MSG_ERR_SPLITMISSINGFROM,
     MSG_ERR_SPLITIDNONE,
