@@ -53,6 +53,10 @@
 #define TRUE (!FALSE)
 #endif
 
+#ifndef NUL
+#define NUL '\0'
+#endif
+
 /* maximum NNTP protocol line length */
 #define NNTP_LINE_MAXLEN 1024
 
@@ -92,7 +96,7 @@ nntp_t *nntp_create(int rfd, int wfd, nntp_io_t *io)
     nntp->wfd = wfd;
     nntp->rl.rl_cnt = 0;
     nntp->rl.rl_bufptr = NULL;
-    nntp->rl.rl_buf[0] = '\0';
+    nntp->rl.rl_buf[0] = NUL;
     nntp_timeout(nntp, 3);
     nntp->kludgeinn441dup = FALSE;
 
@@ -131,24 +135,19 @@ nntp_rc_t nntp_init(nntp_t *nntp)
 
     do {
         if ((rc = nntp_readline(nntp, line, sizeof(line))) != NNTP_OK) {
-            /*fprintf(stderr, "DEBUG: nntp_readline returned ***%d***\n", rc); */
             return rc;
             }
     } while (line[0] == '1');
 
-    /*fprintf(stderr, "DEBUG: nntp_readline got ***%s***\n", line); */
-
     /* prepare for the INN kludge using 441 returns for other things but
      * "Duplicate".  Typical welcome string is "200 host InterNetNews NNRP
-     * server INN 2.3.2 ready (posting * ok)."
+     * server INN 2.3.2 ready (posting ok)."
      */
     if (strncmp(line, "200", 3) == 0 && strstr(line, " INN ") != NULL) {
         nntp->kludgeinn441dup = TRUE;
-        /*fprintf(stderr, "DEBUG: INN kludge activated!\n"); */
     }
     else {
         nntp->kludgeinn441dup = FALSE;
-        /*fprintf(stderr, "DEBUG: no INN kludge!\n"); */
     }
 
     if (strncmp(line, "200", 3) == 0)
@@ -209,10 +208,9 @@ nntp_rc_t nntp_readline(nntp_t *nntp, char *buf, size_t buflen)
         buf[n++] = c;       /* output char into given buffer */
 
     }
-    buf[n] = '\0';          /* string termination */
+    buf[n] = NUL;          /* string termination */
     if (n == (buflen-1)) 
         return NNTP_ERR_OVERFLOW;
-    /*fprintf(stderr, "DEBUG: nntp_readline  <<<%s\n", buf); */
     return NNTP_OK;
 }
 
@@ -224,7 +222,6 @@ nntp_rc_t nntp_writeline(nntp_t *nntp, char *buf)
         return NNTP_ERR_ARG;
     strncpy(tmp, buf, NNTP_LINE_MAXLEN-3);
     strcat(tmp, "\r\n");
-    /*fprintf(stderr, "DEBUG: nntp_writeline >>>%s", tmp); */
     if (nntp->io.write(nntp->wfd, tmp, strlen(tmp)) < 0)
         return NNTP_ERR_SYSTEM;
     return NNTP_OK;
@@ -259,9 +256,13 @@ nntp_rc_t nntp_post(nntp_t *nntp, msg_t *msg)
      *  > POST
      *  < 340 Ok, recommended ID <...>
      *  
-     *  In other words, INN *requires* the use of "MODE READER".
+     *  In other words, INN *requires* the use of "MODE READER". This has been
+     *  verified when INN is configured for expecting both news readers and
+     *  feeders from a given source address. When INN is configured to expect
+     *  readers only for a given source address the use of "MODE READER" is
+     *  optional.
      */
-    *line = '\0';
+    *line = NUL;
     strcat(line, "MODE READER");
     if ((rc = nntp_writeline(nntp, line)) != NNTP_OK)
         return rc;
@@ -285,7 +286,7 @@ nntp_rc_t nntp_post(nntp_t *nntp, msg_t *msg)
      *  < 223 yes, article already known
      *  < 430 no, i don't know the article, yet
      */
-    *line = '\0';
+    *line = NUL;
     strcat(line, "STAT ");
     strcat(line, msg->cpMsgid);
     if ((rc = nntp_writeline(nntp, line)) != NNTP_OK)
@@ -322,10 +323,11 @@ nntp_rc_t nntp_post(nntp_t *nntp, msg_t *msg)
      *  441 Duplicate "Newsgroups" header
      *  441 Required "Subject" header is missing
      *  441 Obsolete "Received" header
+     *  441 Line # too long
      *  
      *  In other words, INN uses 441 for other status messages as well.
      */
-    *line = '\0';
+    *line = NUL;
     strcat(line, "POST");
     if ((rc = nntp_writeline(nntp, line)) != NNTP_OK)
         return rc;
@@ -342,12 +344,11 @@ nntp_rc_t nntp_post(nntp_t *nntp, msg_t *msg)
 
     if ((rc = nntp_readline(nntp, line, sizeof(line))) != NNTP_OK)
         return rc;
-    /*fprintf(stderr, "DEBUG: answer to post = ***%s***, rc = %s\n", line, nntp_error(rc)); */
 
     if (strncmp(line, "240", 3) == 0)
         return NNTP_OK;
 
-    if (nntp->kludgeinn441dup == TRUE) {
+    if (nntp->kludgeinn441dup) {
         if (strncmp(line, "441 435", 7) == 0)
             return NNTP_OK;
     }
@@ -389,7 +390,7 @@ nntp_rc_t nntp_feed(nntp_t *nntp, msg_t *msg)
      *  < 437 Unwanted newsgroup "quux"
      *  < 480 Transfer permission denied
      */
-    *line = '\0';
+    *line = NUL;
     strcat(line, "IHAVE ");
     strcat(line, msg->cpMsgid);
     if ((rc = nntp_writeline(nntp, line)) != NNTP_OK)
@@ -419,7 +420,6 @@ nntp_rc_t nntp_feed(nntp_t *nntp, msg_t *msg)
 
     if ((rc = nntp_readline(nntp, line, sizeof(line))) != NNTP_OK)
         return rc;
-    /*fprintf(stderr, "DEBUG: answer to post = ***%s***, rc = %s\n", line, nntp_error(rc)); */
 
     if (strncmp(line, "235", 3) == 0)
         return NNTP_OK;
